@@ -28,7 +28,10 @@ class RippleError(Exception):
 class ResponseError(RippleError):
     def __init__(self, error_response):
         self.response = error_response
-        RippleError.__init__(self, error_response['error_message'])
+        RippleError.__init__(
+            self,
+            error_response.get('error_exception',
+                               error_response.get('error_message', '')))
 
     def __getitem__(self, item):
         return self.response[item]
@@ -245,7 +248,7 @@ class Client(object):
         data['id'] = self._mkid()
         data = {k:v for k, v in data.items() if v is not None}
 
-        log.debug('>>>>>>>> sending %s', json.dumps(data, indent=2))
+        log.debug('>>>>>>>> sending %s', json.dumps(data, indent=2, cls=RippleEncoder))
         self.conn.send(json.dumps(data, cls=RippleEncoder).encode('utf-8'))
         with self.callbacks_lock:
             self.callbacks[data['id']] = DeferredResponse()
@@ -326,6 +329,21 @@ class Client(object):
         return self.execute(
             "submit", tx_blob=tx_blob, tx_json=tx_json, secret=secret)
 
+    def find_path_once(self, source, destination, destination_amount,
+                       source_currencies=None):
+        """Use the ``ripple_path_find`` API to find a path"""
+        return self.execute(
+            "ripple_path_find", source_account=source,
+            destination_account=destination,
+            destination_amount=destination_amount,
+            source_currencies=source_currencies
+        )
+
+    def path_find(self):
+        # Start an ongoing path-finding process. Uses the ``path_find``
+        # API instead.
+        raise NotImplementedError()
+
 
 class DeferredTransaction(object):
     # TODO: Can we just re-use DeferredResponse? - at least extend it
@@ -434,11 +452,11 @@ class Remote(object):
         self._sequence_cache[account] = info['Sequence']
         return info
 
-    def send_payment(self, destination, amount, account=None):
+    def send_payment(self, destination, amount, account=None, flags=None):
         # Parse the amount
         amount = Amount(amount)
 
-        # Determine the sender
+        # Determine the sender address
         if not account:
             if not self.secret:
                 raise ValueError(
@@ -446,12 +464,34 @@ class Remote(object):
                     'give a secret so one can be derived.')
             account = get_ripple_from_secret(self.secret)
 
+        # If the amount to send does not include an issuer, setting it
+        # to the destination address makes ripple pick one.
+        if amount.currency != 'XRP':
+            if not 'issuer' in amount:
+                amount['issuer'] = destination
+
+        # For non-xrp payments, we need to have a path
+        if amount.currency == 'XRP':
+            paths = None
+        else:
+            pfr = self.client.find_path_once(
+                source=account, destination=destination,
+                destination_amount=amount
+            )
+            if not pfr['alternatives']:
+                raise ValueError('No path found for this payment')
+            paths = pfr['alternatives'][0]['paths_computed']
+
         tx = {
             "TransactionType" : "Payment",
             "Account" : account,
             "Destination" : destination,
             "Amount" : amount,
         }
+        if flags is not None:
+            tx['Flags'] = flags
+        if paths is not None:
+            tx['Paths'] = paths
 
         return self.submit(account, tx)
 
