@@ -110,81 +110,116 @@ class RippleStateEntry(RipplePrimitive):
         raise ValueError('%s is not a party' % account)
 
 
-class Amount(RipplePrimitive):
+class Amount(object):
+    """Represents a Ripple amount. The datastructure behind this will
+    either be a dict with amount/currency/issuer keys, or a natural
+    number, in which case we are dealing with XRP.
 
-    def __init__(self, data, safe=False):
-        if isinstance(data, dict):
-            # Proper currency structure
-            data['value'] = Decimal(data['value'])
-            RipplePrimitive.__init__(self, data)
+    Usage/Features:
+
+    - The properties are used to normalize access.
+    - Supports arithmetic.
+    - Can be initialized with with special formats:
+        Decimal('1') for XRP for drops, for example.
+
+    Note: This ideally should inherit RipplePrimitive, but then we have
+    trouble serializing it to an integer in the XRP case, since
+    RipplePrimitive as a dict, and Python's json module does not provide
+    any hook to change how basic types are handled.
+    """
+
+    def __init__(self, data):
+        # In Ripple data structures, data is either a IOU dict, or
+        # XRP drops in int. We want to allow the developer to init
+        # an Amount object with other values.
+        if isinstance(data, six.string_types):
+            # Treat as XRP, convert to drops, if there is a decimal
+            # point. Otherwise we have to support this as drops, since
+            # rippled often returns drops as string objects.
+            if '.' in data:
+                data = Decimal(data)
+                # Fall-through
+
+        if isinstance(data, Decimal):
+            # If a decimal is given,
+            data = Decimal(data) * xrp_base
+            assert int(data) == data
+            data = int(data)
+
+        self.data = data
+
+    @property
+    def currency(self):
+        if isinstance(self.data, dict):
+            return self.data['currency']
         else:
-            # Parse the amount
-            currency = 'XRP'
-            issuer = None
-            if isinstance(data, int):
-                # A raw xrp number in drops:
-                value = xrp(data)
-            elif isinstance(data, six.string_types):
-                if '.' in data:
-                    value = Decimal(data)
-                elif safe:
-                    value = xrp(data)
-                else:
-                    # For safety, so there can be no confusion.
-                    raise ValueError('When passing a string as amount, it '
-                                     'needs to include a decimal point.')
-            elif isinstance(data, Decimal):
-                # Use as provided
-                value = data
-            else:
-                # TODO: Still need to support IOUs
-                raise ValueError('cannot handle amount: %s' % data)
+            return 'XRP'
 
-            RipplePrimitive.__init__(self, {
-                'currency': currency,
-                'issuer': issuer,
-                'value': value
-            })
-
-    def __json__(self):
-        if self.currency == 'XRP':
-            in_drops = (self.value * xrp_base)
-            assert int(in_drops) == in_drops
-            return int((self.value * xrp_base))
+    @property
+    def issuer(self):
+        if isinstance(self.data, dict):
+            return self.data['issuer']
         else:
-            return RipplePrimitive.__json__(self)
+            return None
+
+    def _get_value(self):
+        if isinstance(self.data, dict):
+            return Decimal(self.data['value'])
+        else:
+            return xrp(self.data)
+    def _set_value(self, v):
+        if isinstance(self.data, dict):
+            self.data['value'] = '%s' % v
+        else:
+            self.data = int(Decimal(v) * xrp_base)
+    value = property(_get_value, _set_value)
+
+    def copy(self, new_value=None):
+        copy = Amount(self.data.copy() if isinstance(self.data, dict) else self.data)
+        if new_value is not None:
+            copy.value = new_value
+        return copy
 
     def __unicode__(self):
         return '%s' % self.value
 
+    def __json__(self):
+        return self.data
+
+    def __contains__(self, item):
+        return item in self.data
+
+    def __getitem__(self, item):
+        return self.data[item]
+
+    def __setitem__(self, key, value):
+        self.data[key] = value
+
     def _assert_compat_other(self, other):
         """For arithmetic with the Amount class, check that the ``other``
         object can be handled.
         """
-        if isinstance(other, unicode):
-            return Decimal(unicode)
+        if isinstance(other, str):
+            return Decimal(other)
         if isinstance(other, Decimal):
             return other
         assert other.currency == self.currency
-        return other['value']
+        return other.value
 
     def __add__(self, other):
         other_value = self._assert_compat_other(other)
-        result = self.copy()
-        result['value'] = self['value'] + other_value
-        return result
+        new_value = self.value + other_value
+        return self.copy(new_value)
 
     def __sub__(self, other):
         other_value = self._assert_compat_other(other)
-        result = self.copy()
-        result['value'] = self['value'] - other_value
-        return result
+        new_value = self.value - other_value
+        return self.copy(new_value)
 
     def __div__(self, other):
         other_value = self._assert_compat_other(other)
-        result = self.copy()
-        result['value'] = self['value'] / other_value
-        return result
+        new_value = self.value / other_value
+        return self.copy(new_value)
 
     def __rdiv__(self, other):
         return Amount.__truediv__(other, self)
@@ -440,7 +475,9 @@ class PaymentTransaction(Transaction):
         amount received.
         """
         if self.meta.get('DeliveredAmount'):
-            amount = self.meta.DeliveredAmount.value
+            amount = self.meta.DeliveredAmount.value \
+                if isinstance(self.meta.DeliveredAmount, dict) \
+                else xrp(self.meta.DeliveredAmount)
         else:
             amount = self.Amount.value \
                 if isinstance(self.Amount, dict) else xrp(self.Amount)
